/***************************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

***************************************************************************
Copyright (C) 2005 by 
Pan Wojtas (Wojtek Sulewski)
wojteksulewski <at> op.pl
gg: 2087202

***************************************************************************/


#include <qcheckbox.h>
#include <qspinbox.h>

#include "debug.h"
#include "config_dialog.h"
#include "chat.h"

#include "osd_manager.h"

OSDManager::OSDManager()
{
	height = 0;
}

OSDManager::~OSDManager()
{
	deleteAll();
}

void OSDManager::addOSD(const QString& text, const QPixmap& pixmap, const QFont &font, const QColor &color, const QColor &bgcolor, unsigned int timeout, bool translucency, double translucency_level, int x, int y, const UinsList &senders)
{
	kdebugf();
	
	unsigned int new_osd = config_file.readUnsignedNumEntry("osdhints","NewOSD", 0);
	if (new_osd == 5)
		deleteAll();
	
	widgets.append(new OSDWidget(0));
	unsigned int id = widgets.count() - 1;
	kdebugm(KDEBUG_INFO, "NEW OSDWidget id: %d\n", id);
	kdebugm(KDEBUG_INFO, "height: %d\n", height);
	kdebugm(KDEBUG_INFO, "height: %d\n", new_osd);
	
	widgets.at(id)->setTextColor(color);
	widgets.at(id)->setBackgroundColor(bgcolor);
	widgets.at(id)->setDuration(timeout);
	widgets.at(id)->setPixmap(pixmap);
	widgets.at(id)->setText(text);
	widgets.at(id)->setTranslucency(translucency);
	widgets.at(id)->setTranslucencyLevel(translucency_level);

	widgets.at(id)->setX(x);
	
	if (new_osd == 0)
		widgets.at(id)->setY(y - height);
	else if (new_osd == 2)
		widgets.at(id)->setY(y + height);
	else
		widgets.at(id)->setY(y);
		
	widgets.at(id)->setFont(font);
	widgets.at(id)->setUin(senders);
		

	widgets.at(id)->setID(id);
	
	if(new_osd == 1 || new_osd == 3)
	{
		FOREACH(i, widgets)
		{
			(*i)->hide();
		}
	}
	
	widgets.at(id)->show();
	
	unsigned int separator = config_file.readUnsignedNumEntry("osdhints","OSD_separator");
	
	if (height > 0 && new_osd == 3)
	{
		for (unsigned int iter = 0; iter < widgets.count() - 1; iter++)
		{
			widgets.at(iter)->setY(widgets.at(iter)->m_y - widgets.at(id)->m_height - separator);
			widgets.at(iter)->show();
			widgets.at(iter)->QWidget::show();
		}
	}
	else if(height > 0 && new_osd == 1)
	{
		for (unsigned int iter = 0; iter < widgets.count() - 1; iter++)
		{
			widgets.at(iter)->setY(widgets.at(iter)->m_y + widgets.at(id)->m_height + separator);
			widgets.at(iter)->show();
			widgets.at(iter)->QWidget::show();
		}
	}

	widgets.at(id)->QWidget::show();
	
 	height += widgets.at(id)->m_height + separator;
	kdebugm(KDEBUG_INFO, "height: %d\n",  height);

	connect(widgets.at(id), SIGNAL(timeout(unsigned int, bool)), this, SLOT(timeout(unsigned int, bool)));
	connect(widgets.at(id), SIGNAL(deleteAll(void)), this, SLOT(deleteAll(void)));

	kdebugf2();
}

void OSDManager::timeout(unsigned int id, bool chat)
{
	kdebugf();

	kdebugm(KDEBUG_INFO, "timeout: OSDWidget id: %d\n", id);
	unsigned int def_y = config_file.readUnsignedNumEntry("osdhints","OSD_y");
	unsigned int separator = config_file.readUnsignedNumEntry("osdhints","OSD_separator");
	unsigned int new_osd = config_file.readUnsignedNumEntry("osdhints","NewOSD");

	height = 0;
	UinsList chat_uin = widgets.at(id)->m_uin;
	

	QPtrList<OSDWidget> new_widgets;
	
	FOREACH(it, widgets)
	{
		// jeli jest pokazany, to przepisuje. sprawdza czy jest chat - wtedy zamyka wszystkie z tym uinem
		if ((*it)->isShown() && !(chat && (*it)->m_uin == chat_uin))
		{
			(*it)->hide();
			if (new_osd == 0)
				(*it)->setY(def_y - height);
			else if (new_osd == 2)
				(*it)->setY(def_y + height);
			else if (new_osd == 5)
			{
				deleteAll();
				(*it)->setY(def_y + height);
				new_widgets.append((*it));
				break;
			}
			new_widgets.append((*it));

			height += (*it)->m_height + separator;
		}
		else
			(*it)->close();

	}
	
	if (new_osd == 3)
	{
		int height_tmp = height;
		FOREACH(it, new_widgets)
		{
			height_tmp -= (*it)->m_height + separator;
			(*it)->setY(def_y - height_tmp);
		}
	}
	else if (new_osd == 1)
	{
		int height_tmp = height;
		FOREACH(it, new_widgets)
		{
			height_tmp -= (*it)->m_height + separator;
			(*it)->setY(def_y + height_tmp);
		}
	}

	
	widgets.clear();
	widgets = new_widgets;
	unsigned int new_id = 0;
	FOREACH(it, widgets)
	{
		(*it)->setID(new_id);
		(*it)->show();
		(*it)->QWidget::show();
		new_id++;
	}

	kdebugf2();
}

void OSDManager::deleteAll(void)
{
	kdebugf();

	height = 0;

	CONST_FOREACH(it, widgets)
	{
		(*it)->close();
	}
	widgets.clear();

	kdebugf2();
}

/**

	SLOTY dla notify

**/

void OSDManager::newChat(const UinsList &senders, const QString& msg, time_t /*time*/)
{
	kdebugf();

	QString text;
	QPixmap pixmap;
	QFont font = config_file.readFontEntry("osdhints", "OSDNewChat_font");
	QColor text_color;
	QColor bg_color;
	unsigned int timeout = config_file.readUnsignedNumEntry("osdhints", "OSDNewChat_timeout", 3600);
	bool translucency = config_file.readBoolEntry("osdhints", "Use translucency", false);
	double translucency_level = config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01;
	int x = config_file.readNumEntry("osdhints", "OSD_x", 300);
	int y = config_file.readNumEntry("osdhints", "OSD_y", 300);
	
// zdjcie
	QString photo_path = ggPath(parse(config_file.readEntry("osdhints","PhotoSyntax_chat"), userlist.byUinValue(senders[0]), true));
	if (config_file.readBoolEntry("osdhints", "UsePhoto_chat", false) && QFile::exists(photo_path))
		pixmap = icons_manager.loadIcon(photo_path);
	else
		pixmap = icons_manager.loadIcon(config_file.readEntry("osdhints", "OSDNewChat_icon"));

// wczytanie kolorw
	if (config_file.readBoolEntry("osdhints", "OSDNewChat_def_colors", true))
	{
		text_color = def_text_color;
		bg_color = def_bg_color;
	}
	else
	{
		text_color = config_file.readColorEntry("osdhints", "OSDNewChat_fgcolor");
		bg_color = config_file.readColorEntry("osdhints", "OSDNewChat_bgcolor");
	}

// skadnia
	if (config_file.readBoolEntry("osdhints", "UseSyntax_chat", true))
		text = parse(config_file.readEntry("osdhints","Syntax_chat"), userlist.byUinValue(senders[0]), true);
	else
		text = tr("Chat with <b>%1</b>").arg(QStyleSheet::escape(userlist.byUinValue(senders[0]).altNick()));
	
// cytowanie
	if (config_file.readBoolEntry("osdhints", "ShowContentMessage", true))
	{
		unsigned int citeSign=config_file.readUnsignedNumEntry("osdhints","CiteSign", 100);
		
		if (msg.length() <= citeSign)
			text += "<BR><small>" + msg + "</small>";
		else
			text += "<BR><small>" + msg.left(citeSign) + "..." + "</small>";
	}

	addOSD( text, pixmap, font, text_color, bg_color, timeout, translucency, translucency_level, x, y, senders);

	kdebugf2();
}

void OSDManager::newMessage(const UinsList &senders, const QString& msg, time_t /*time*/, bool &/*grab*/)
{
	kdebugf();
	
	QString text;
	QPixmap pixmap;
	QFont font = config_file.readFontEntry("osdhints", "OSDNewMessage_font");
	QColor text_color;
	QColor bg_color;
	unsigned int timeout = config_file.readUnsignedNumEntry("osdhints", "OSDNewMessage_timeout", 3600);
	bool translucency = config_file.readBoolEntry("osdhints", "Use translucency", false);
	double translucency_level = config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01;
	int x = config_file.readNumEntry("osdhints", "OSD_x", 300);
	int y = config_file.readNumEntry("osdhints", "OSD_y", 300);
	
	

	Chat* chat=chat_manager->findChatByUins(senders);
	if(chat->isActiveWindow())
		return;
	if (chat==NULL)
	{
		kdebugmf(KDEBUG_ERROR|KDEBUG_FUNCTION_END, "end: chat==NULL!\n");
		return;
	}

// zdjcie
	QString photo_path = ggPath(parse(config_file.readEntry("osdhints","PhotoSyntax_chat"), userlist.byUinValue(senders[0]), true));
	if (config_file.readBoolEntry("osdhints", "UsePhoto_chat", false) && QFile::exists(photo_path))
		pixmap = icons_manager.loadIcon(photo_path);
	else
		pixmap = icons_manager.loadIcon(config_file.readEntry("osdhints", "OSDNewMessage_icon"));

// wczytanie kolorw
	if (config_file.readBoolEntry("osdhints", "OSDNewMessage_def_colors", true))
	{
		text_color = def_text_color;
		bg_color = def_bg_color;
	}
	else
	{
		text_color = config_file.readColorEntry("osdhints", "OSDNewMessage_fgcolor");
		bg_color = config_file.readColorEntry("osdhints", "OSDNewMessage_bgcolor");
	}

// skadnia
	if (config_file.readBoolEntry("osdhints", "UseSyntax_chat", true))
		text = parse(config_file.readEntry("osdhints","Syntax_chat"), userlist.byUinValue(senders[0]), true);
	else
		text = tr("New message from <b>%1</b>").arg(QStyleSheet::escape(userlist.byUinValue(senders[0]).altNick()));
	
// cytowanie
	if (config_file.readBoolEntry("osdhints", "ShowContentMessage", true))
	{
		unsigned int citeSign=config_file.readUnsignedNumEntry("osdhints","CiteSign", 100);
		
		if (msg.length() <= citeSign)
			text += "<BR><small>" + msg + "</small>";
		else
			text += "<BR><small>" + msg.left(citeSign) + "..." + "</small>";
	}

	addOSD( text, pixmap, font, text_color, bg_color, timeout, translucency, translucency_level, x, y, senders);

	kdebugf2();
}

void OSDManager::userChangedStatusToAvailable(const UserListElement &ule)
{
	kdebugf();
	
	UinsList ulist;
	if (config_file.readBoolEntry("osdhints", "OpenChatOnClick", false))
		ulist.append(ule.uin());
	
	QString name;
	bool has_description = ule.status().hasDescription();
	if (has_description)
		name = "OSDOnlineD_";
	else
		name = "OSDOnline_";


	QString text;
	QPixmap pixmap;
	QFont font = config_file.readFontEntry("osdhints", name + "font");
	QColor text_color;
	QColor bg_color;
	unsigned int timeout = config_file.readUnsignedNumEntry("osdhints", name + "timeout", 3600);
	bool translucency = config_file.readBoolEntry("osdhints", "Use translucency", false);
	double translucency_level = config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01;
	int x = config_file.readNumEntry("osdhints", "OSD_x", 300);
	int y = config_file.readNumEntry("osdhints", "OSD_y", 300);
	
// zdjcie
	QString photo_path = ggPath(parse(config_file.readEntry("osdhints","PhotoSyntax_status"), ule, true));
	if (config_file.readBoolEntry("osdhints", "UsePhoto_status", false) && QFile::exists(photo_path))
		pixmap = icons_manager.loadIcon(photo_path);
	else
		pixmap = icons_manager.loadIcon(config_file.readEntry("osdhints", name + "icon"));

// wczytanie kolorw
	if (config_file.readBoolEntry("osdhints", name + "def_colors", true))
	{
		text_color = def_text_color;
		bg_color = def_bg_color;
	}
	else
	{
		text_color = config_file.readColorEntry("osdhints", name + "fgcolor");
		bg_color = config_file.readColorEntry("osdhints", name + "bgcolor");
	}

// skadnia
	if (config_file.readBoolEntry("osdhints", "UseSyntax_status", true))
		text = parse(config_file.readEntry("osdhints","Syntax_status"), ule, true);
	else
		text = tr("<b>%1</b> is available").arg(QStyleSheet::escape(ule.altNick()));
	
// cytowanie
	if (has_description)
	{
		text += QString("<BR><small>%1</small>").arg(QStyleSheet::escape(ule.status().description()));
	}

	addOSD( text, pixmap, font, text_color, bg_color, timeout, translucency, translucency_level, x, y, ulist);

	kdebugf2();
}

void OSDManager::userChangedStatusToBusy(const UserListElement &ule)
{
	kdebugf();
	
	UinsList ulist;
	if (config_file.readBoolEntry("osdhints", "OpenChatOnClick", false))
		ulist.append(ule.uin());
	
	QString name;
	bool has_description = ule.status().hasDescription();
	if (has_description)
		name = "OSDBusyD_";
	else
		name = "OSDBusy_";


	QString text;
	QPixmap pixmap;
	QFont font = config_file.readFontEntry("osdhints", name + "font");
	QColor text_color;
	QColor bg_color;
	unsigned int timeout = config_file.readUnsignedNumEntry("osdhints", name + "timeout", 3600);
	bool translucency = config_file.readBoolEntry("osdhints", "Use translucency", false);
	double translucency_level = config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01;
	int x = config_file.readNumEntry("osdhints", "OSD_x", 300);
	int y = config_file.readNumEntry("osdhints", "OSD_y", 300);
	
// zdjcie
	QString photo_path = ggPath(parse(config_file.readEntry("osdhints","PhotoSyntax_status"), ule, true));
	if (config_file.readBoolEntry("osdhints", "UsePhoto_status", false) && QFile::exists(photo_path))
		pixmap = icons_manager.loadIcon(photo_path);
	else
		pixmap = icons_manager.loadIcon(config_file.readEntry("osdhints", name + "icon"));

// wczytanie kolorw
	if (config_file.readBoolEntry("osdhints", name + "def_colors", true))
	{
		text_color = def_text_color;
		bg_color = def_bg_color;
	}
	else
	{
		text_color = config_file.readColorEntry("osdhints", name + "fgcolor");
		bg_color = config_file.readColorEntry("osdhints", name + "bgcolor");
	}

// skadnia
	if (config_file.readBoolEntry("osdhints", "UseSyntax_status", true))
		text = parse(config_file.readEntry("osdhints","Syntax_status"), ule, true);
	else
		text = tr("<b>%1</b> is busy").arg(QStyleSheet::escape(ule.altNick()));
	
// cytowanie
	if (has_description)
	{
		text += QString("<BR><small>%1</small>").arg(QStyleSheet::escape(ule.status().description()));
	}

	addOSD( text, pixmap, font, text_color, bg_color, timeout, translucency, translucency_level, x, y, ulist);

	kdebugf2();
}

void OSDManager::userChangedStatusToInvisible(const UserListElement &ule)
{
	kdebugf();
	
	UinsList ulist;
	if (config_file.readBoolEntry("osdhints", "OpenChatOnClick", false))
		ulist.append(ule.uin());
	
	QString name;
	bool has_description = ule.status().hasDescription();
	if (has_description)
		name = "OSDInvisibleD_";
	else
		name = "OSDInvisible_";


	QString text;
	QPixmap pixmap;
	QFont font = config_file.readFontEntry("osdhints", name + "font");
	QColor text_color;
	QColor bg_color;
	unsigned int timeout = config_file.readUnsignedNumEntry("osdhints", name + "timeout", 3600);
	bool translucency = config_file.readBoolEntry("osdhints", "Use translucency", false);
	double translucency_level = config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01;
	int x = config_file.readNumEntry("osdhints", "OSD_x", 300);
	int y = config_file.readNumEntry("osdhints", "OSD_y", 300);
	
// zdjcie
	QString photo_path = ggPath(parse(config_file.readEntry("osdhints","PhotoSyntax_status"), ule, true));
	if (config_file.readBoolEntry("osdhints", "UsePhoto_status", false) && QFile::exists(photo_path))
		pixmap = icons_manager.loadIcon(photo_path);
	else
		pixmap = icons_manager.loadIcon(config_file.readEntry("osdhints", name + "icon"));

// wczytanie kolorw
	if (config_file.readBoolEntry("osdhints", name + "def_colors", true))
	{
		text_color = def_text_color;
		bg_color = def_bg_color;
	}
	else
	{
		text_color = config_file.readColorEntry("osdhints", name + "fgcolor");
		bg_color = config_file.readColorEntry("osdhints", name + "bgcolor");
	}

// skadnia
	if (config_file.readBoolEntry("osdhints", "UseSyntax_status", true))
		text = parse(config_file.readEntry("osdhints","Syntax_status"), ule, true);
	else
		text = tr("<b>%1</b> is invisible").arg(QStyleSheet::escape(ule.altNick()));
	
// cytowanie
	if (has_description)
	{
		text += QString("<BR><small>%1</small>").arg(QStyleSheet::escape(ule.status().description()));
	}

	addOSD( text, pixmap, font, text_color, bg_color, timeout, translucency, translucency_level, x, y, ulist);

	kdebugf2();
}

void OSDManager::userChangedStatusToNotAvailable(const UserListElement &ule)
{
	kdebugf();
	
	UinsList ulist;
	if (config_file.readBoolEntry("osdhints", "OpenChatOnClick", false))
		ulist.append(ule.uin());
	
	QString name;
	bool has_description = ule.status().hasDescription();
	if (has_description)
		name = "OSDOfflineD_";
	else
		name = "OSDOffline_";


	QString text;
	QPixmap pixmap;
	QFont font = config_file.readFontEntry("osdhints", name + "font");
	QColor text_color;
	QColor bg_color;
	unsigned int timeout = config_file.readUnsignedNumEntry("osdhints", name + "timeout", 3600);
	bool translucency = config_file.readBoolEntry("osdhints", "Use translucency", false);
	double translucency_level = config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01;
	int x = config_file.readNumEntry("osdhints", "OSD_x", 300);
	int y = config_file.readNumEntry("osdhints", "OSD_y", 300);
	
// zdjcie
	QString photo_path = ggPath(parse(config_file.readEntry("osdhints","PhotoSyntax_status"), ule, true));
	if (config_file.readBoolEntry("osdhints", "UsePhoto_status", false) && QFile::exists(photo_path))
		pixmap = icons_manager.loadIcon(photo_path);
	else
		pixmap = icons_manager.loadIcon(config_file.readEntry("osdhints", name + "icon"));

// wczytanie kolorw
	if (config_file.readBoolEntry("osdhints", name + "def_colors", true))
	{
		text_color = def_text_color;
		bg_color = def_bg_color;
	}
	else
	{
		text_color = config_file.readColorEntry("osdhints", name + "fgcolor");
		bg_color = config_file.readColorEntry("osdhints", name + "bgcolor");
	}

// skadnia
	if (config_file.readBoolEntry("osdhints", "UseSyntax_status", true))
		text = parse(config_file.readEntry("osdhints","Syntax_status"), ule, true);
	else
		text = tr("<b>%1</b> is not available").arg(QStyleSheet::escape(ule.altNick()));
	
// cytowanie
	if (has_description)
	{
		text += QString("<BR><small>%1</small>").arg(QStyleSheet::escape(ule.status().description()));
	}

	addOSD( text, pixmap, font, text_color, bg_color, timeout, translucency, translucency_level, x, y, ulist);

	kdebugf2();
}

void OSDManager::connectionError(const QString &message)
{
	kdebugf();
	
	UinsList ulist;
	QColor text;
	QColor bg;
	if (config_file.readBoolEntry("osdhints", "OSDError_def_colors", true))
	{
		text = def_text_color;
		bg = def_bg_color;
	}
	else
	{
		text = config_file.readColorEntry("osdhints", "OSDError_fgcolor");
		bg = config_file.readColorEntry("osdhints", "OSDError_bgcolor");
	}
	
	addOSD(tr("<b>Error:</b> %1").arg(message),
	       icons_manager.loadIcon(config_file.readEntry("osdhints", "OSDError_icon")),
	       config_file.readFontEntry("osdhints", "OSDError_font"),
	       text,
	       bg,
	       config_file.readUnsignedNumEntry("osdhints", "OSDError_timeout", 3000),
	       config_file.readBoolEntry("osdhints", "Use translucency", false),
	       config_file.readNumEntry("osdhints", "Translucency level", 80) * 0.01,
	       config_file.readNumEntry("osdhints", "OSD_x", 300),
	       config_file.readNumEntry("osdhints", "OSD_y", 300),
	       ulist);
	kdebugf2();
}

void OSDManager::message(const QString &from, const QString &msg, const QMap<QString, QVariant> *parameters, const UserListElement *ule)
{
	kdebugf();
	UinsList uinslist;
	if (ule != NULL)
		uinslist.append(ule->uin());

	QString msg2;
	QPixmap pixmap;
	QFont font;
	QColor text;
	QColor bg;
	unsigned int timeout;
	unsigned int translucency_level;
	unsigned int x;
	unsigned int y;
	bool translucency;
	bool ok, ok2, ok3, ok4;

	QMap<QString, QVariant>::const_iterator sit=(*parameters).find("Translucency");
	QMap<QString, QVariant>::const_iterator end=(*parameters).end();
	if (parameters!=NULL)
	{
		QMap<QString, QVariant>::const_iterator end=(*parameters).end();
		
		pixmap=(*parameters)["Pixmap"].toPixmap();
		font=(*parameters)["Font"].toFont();
		text=(*parameters)["Foreground color"].toColor();
		bg=(*parameters)["Background color"].toColor();
		timeout=(*parameters)["Timeout"].toUInt(&ok);
		
		if (sit!=end)
			translucency=sit.data().toBool();
		
		translucency_level=(*parameters)["Translucency level"].toUInt(&ok2);
		x=(*parameters)["X"].toUInt(&ok3);
		y=(*parameters)["Y"].toUInt(&ok4);
	}
	
	if (!text.isValid())
	{
		if (config_file.readBoolEntry("osdhints", "OSDMessage_def_colors", true))
		{
			text = def_text_color;
		}
		else
		{
			text = config_file.readColorEntry("osdhints", "OSDMessage_fgcolor");
		}
	}
	if (!bg.isValid())
	{
		if (config_file.readBoolEntry("osdhints", "OSDMessage_def_colors", true))
		{
			bg = def_bg_color;
		}
		else
		{
			bg = config_file.readColorEntry("osdhints", "OSDMessage_bgcolor");
		}
		
	}

	
	if (pixmap.isNull())
		pixmap=icons_manager.loadIcon(config_file.readEntry("osdhints", "OSDMessage_icon"));
	if (font==qApp->font())
		font=config_file.readFontEntry("osdhints", "OSDMessage_font");
	if (!ok)
		timeout=config_file.readUnsignedNumEntry("osdhints", "OSDMessage_timeout", 3000);
	if (sit==end)
		translucency=config_file.readBoolEntry("osdhints", "Use translucency", false);
	if (!ok2)
		translucency_level=config_file.readNumEntry("osdhints", "Translucency level", 80);
	if (!ok3)
		x=config_file.readNumEntry("osdhints", "OSD_x", 300);
	if (!ok4)
		y=config_file.readNumEntry("osdhints", "OSD_y", 300);
		
	if (!from.isEmpty())
		msg2=narg(tr("From <b>%1</b>: %2"), from, msg);
	else
		msg2=msg;

	addOSD(msg2, pixmap, font, text, bg, timeout, translucency,  0.01 * translucency_level, x, y, uinslist);

	kdebugf2();
}
