/***************************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

***************************************************************************
Copyright (C) 2005 by
Pan Wojtas (Wojtek Sulewski)
wojteksulewski <at> op.pl
gg: 2087202

*/
/*
This file is based on osd from AmaroK
 * osd.cpp:   Shows some text in a pretty way independent to the WM
 * begin:     Fre Sep 26 2003
 * copyright: (C) 2004 Christian Muehlhaeuser <chris@chris.de>
 *            (C) 2004 Seb Ruiz <seb100@optusnet.com.au>
 *            (C) 2004, 2005 Max Howell
*/

#include <qtimer.h>
#include <qbitmap.h>
#include <qregexp.h>
#include <qsimplerichtext.h>
#include <qvbuttongroup.h>


#include "debug.h"
#include "chat.h"
#include "config_dialog.h"

#include "osd_widget.h"

const QColorGroup c = QApplication::palette().active();
QColor def_text_color = c.highlightedText();
QColor def_bg_color = c.highlight();

OSDWidget::OSDWidget( QWidget *parent, const char *name )
		: QHBox( parent, name, WType_TopLevel | WNoAutoErase | WStyle_Customize | WX11BypassWM | WStyle_StaysOnTop | WDestructiveClose)
		, m_timer( new QTimer( this ) )
{
	kdebugf();
	
	setFocusPolicy( NoFocus );
	
	margin_left = new QWidget(this, 0, WDestructiveClose);
	l_pixmap = new QLabel(this, 0, WDestructiveClose);
	margin_center = new QWidget(this, 0, WDestructiveClose);
	
	l_text = new QLabel(this, 0, WDestructiveClose);
	l_text->setTextFormat(Qt::RichText);
	
	margin_right = new QWidget(this, 0, WDestructiveClose);
	
	m_text.replace(QRegExp(" "), "&nbsp;");
	m_text.replace(QRegExp("\n"), "<BR>");
	
	
	unsetColors();

	connect( m_timer, SIGNAL(timeout()), this, SLOT(timeout()));
	installEventFilter(this);
	
	kdebugf2();
}

void OSDWidget::OSDWidget::show()
{
	kdebugf();

	const unsigned int M = fontMetrics().width('x');
	
	l_text->setText(m_text);
	l_text->show();

	if (!m_pixmap.isNull())
	{
		l_pixmap->setPixmap(m_pixmap);
		l_pixmap->setAlignment( AlignRight | AlignVCenter );
		l_pixmap->show();
		
		l_text->setAlignment( AlignLeft | AlignVCenter );
	}
	else
	{
		l_pixmap->setText("");
		l_text->setAlignment( AlignCenter );
	}
	
	QRect newGeometry = determineMetrics(M);
	setGeometry( newGeometry );
	
	render( M, newGeometry );
	
//	QWidget::show();
	
//	setGeometry( newGeometry );
	

//	kdebugm(KDEBUG_PANIC, "\n\n\n\n%d\n\n\n", geometry().width());

	if( m_duration ) //duration 0 -> stay forever
		m_timer->start( m_duration, true ); //calls hide()

	kdebugf2();
}

QRect OSDWidget::determineMetrics(unsigned int M)
{
	kdebugf();

	margin_left->setFixedWidth(M);
	if (!m_pixmap.isNull())
	{
		margin_left->setFixedWidth(M);
		margin_center->setFixedWidth(2*M);
		l_pixmap->setFixedWidth(m_pixmap.width());
	}
	else
	{
	// margin_left - 2 bo dwa pozostae maj po 1
		margin_left->setFixedWidth(M - 2);
		margin_center->setFixedWidth(1);
		l_pixmap->setFixedWidth(1);
	}
	margin_right->setFixedWidth(M);
	
	if (!m_pixmap.isNull())
		m_width = 4*margin_left->width() + l_pixmap->sizeHint().width() + l_text->sizeHint().width();
	else
		m_width = margin_left->width() + margin_center->width() + l_pixmap->width() + l_text->sizeHint().width() + margin_right->width();
	
	if (!m_pixmap.isNull())
	{
		m_pixmap.height()>l_text->sizeHint().height() ? m_height = m_pixmap.height() : m_height = l_text->sizeHint().height();
	}
	else
		m_height = l_text->sizeHint().height();
		 
	m_height += M;
	
// ustawia pozycj dymka
	QPoint newPos( 0, 0 );
	const QRect screen = QApplication::desktop()->screenGeometry( 0 );
	QRect rect(newPos.x(), newPos.y(), m_width, m_height);
	const QSize newSize = rect.size();
	int corner;
	if (ConfigDialog::dialogOpened())
		corner = ConfigDialog::getVButtonGroup("osdhints", "For corner:")->selectedId();
	else
		corner = config_file.readNumEntry("osdhints", "Corner");
	
	switch (corner)
	{
		case 1:		// Left bottom
			newPos.rx() = m_x;
			newPos.ry() = m_y - rect.height();
			break;
		case 2:		// Right top
			newPos.rx() = m_x - rect.width();
			newPos.ry() = m_y;
			break;
		case 3:		// Right bottom
			newPos.rx() = m_x - rect.width();
			newPos.ry() = m_y - rect.height();
			break;
		default:	// Left top
			newPos.rx() = m_x;
			newPos.ry() = m_y;
	}
		
		kdebugf2();
	
	//ensure we don't dip below the screen
	if ( newPos.x() + newSize.width() > screen.width() )
		newPos.rx() = screen.width() - newSize.width();
	else if ( newPos.x() < 0 )
		newPos.rx() = 0;
	if ( newPos.y() + newSize.height() > screen.height() )
		newPos.ry() = screen.height() - newSize.height();
	else if ( newPos.y() < 0 )
		newPos.ry() = 0;
		
	// correct for screen position
	newPos += screen.topLeft();

	return QRect(newPos.x(), newPos.y(), m_width, m_height);
}

void OSDWidget::render( const uint M, const QRect &rect )
{
	kdebugf();


	QPoint point;
	QRect a( point, rect.size() );
	
// zaokrglone rogi
	// From qt sources
	const uint xround = (M * 200) / rect.width();
	const uint yround = (M * 200) / rect.height();

	{   /// apply the mask
		static QBitmap mask;
	
		mask.resize( rect.size() );
		mask.fill( Qt::black );
	
		QPainter p( &mask );
		p.setBrush( Qt::white );
		p.drawRoundRect( a, xround, yround );
		setMask( mask );
	}

// przezroczysto
	if( m_translucency )
	{
		int shift = 0;
	// margin_left
		QPixmap screenshot_l = QPixmap::grabWindow( qt_xrootwin(),
			rect.x(), rect.y(),
			margin_left->width(), m_height );
		background_l = fade(screenshot_l.convertToImage(), m_translucency_level, paletteBackgroundColor());
		background_l_l = fade(screenshot_l.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
		margin_left->setPaletteBackgroundPixmap(background_l);
		shift += margin_left->width();
		
	// pixmap
	
// TODO: mona si pozby ifa prze sizeHint()
		if (!m_pixmap.isNull())
		{
			QPixmap screenshot_p = QPixmap::grabWindow( qt_xrootwin(),
				rect.x() + shift, rect.y(),
				m_pixmap.width(), m_height );
			background_p = fade(screenshot_p.convertToImage(), m_translucency_level, paletteBackgroundColor());
			background_p_l = fade(screenshot_p.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
			l_pixmap->setPaletteBackgroundPixmap(background_p);
			shift += m_pixmap.width();
			
	// center
			QPixmap screenshot_c = QPixmap::grabWindow( qt_xrootwin(),
				rect.x() + shift, rect.y(),
				margin_center->width(), m_height );
			background_c = fade(screenshot_c.convertToImage(), m_translucency_level, paletteBackgroundColor());
			background_c_l = fade(screenshot_c.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
			margin_center->setPaletteBackgroundPixmap(background_c);
			shift += margin_center->width();
		}
		else
		{
			QPixmap screenshot_p = QPixmap::grabWindow( qt_xrootwin(),
				rect.x() + shift, rect.y(),
				1, m_height );
			background_p = fade(screenshot_p.convertToImage(), m_translucency_level, paletteBackgroundColor());
			background_p_l = fade(screenshot_p.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
			l_pixmap->setPaletteBackgroundPixmap(background_p);
			shift += 1;
			
	// center
			QPixmap screenshot_c = QPixmap::grabWindow( qt_xrootwin(),
				rect.x() + shift, rect.y(),
				margin_center->width(), m_height );
			background_c = fade(screenshot_c.convertToImage(), m_translucency_level, paletteBackgroundColor());
			background_c_l = fade(screenshot_c.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
			margin_center->setPaletteBackgroundPixmap(background_c);
			shift += 1;
		}
	// text
		QPixmap screenshot_t = QPixmap::grabWindow( qt_xrootwin(),
			rect.x() + shift, rect.y(),
			l_text->sizeHint().width(), m_height );
		background_t = fade(screenshot_t.convertToImage(), m_translucency_level, paletteBackgroundColor());
		background_t_l = fade(screenshot_t.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
		l_text->setPaletteBackgroundPixmap(background_t);
		shift += l_text->sizeHint().width();
	// right
		QPixmap screenshot_r = QPixmap::grabWindow( qt_xrootwin(),
			rect.x() + shift, rect.y(),
			margin_right->width(), m_height );
		background_r = fade(screenshot_r.convertToImage(), m_translucency_level, paletteBackgroundColor());
		background_r_l = fade(screenshot_r.convertToImage(), m_translucency_level, paletteBackgroundColor().light(160));
		margin_right->setPaletteBackgroundPixmap(background_r);
	}
	else
	{
		margin_left->setPaletteBackgroundColor(paletteBackgroundColor());
		l_pixmap->setPaletteBackgroundColor(paletteBackgroundColor());
		margin_center->setPaletteBackgroundColor(paletteBackgroundColor());
		l_text->setPaletteBackgroundColor(paletteBackgroundColor());
		margin_right->setPaletteBackgroundColor(paletteBackgroundColor());
	}

	kdebugf2();
}

void OSDWidget::enter(void)
{
	kdebugf();
	
	if( m_translucency )
	{
		margin_left->setPaletteBackgroundPixmap(background_l_l);
		l_pixmap->setPaletteBackgroundPixmap(background_p_l);
		margin_center->setPaletteBackgroundPixmap(background_c_l);
		l_text->setPaletteBackgroundPixmap(background_t_l);
		margin_right->setPaletteBackgroundPixmap(background_r_l);
	}
	else
	{
		margin_left->setPaletteBackgroundColor(paletteBackgroundColor().light(160));
		l_pixmap->setPaletteBackgroundColor(paletteBackgroundColor().light(160));
		margin_center->setPaletteBackgroundColor(paletteBackgroundColor().light(160));
		l_text->setPaletteBackgroundColor(paletteBackgroundColor().light(160));
		margin_right->setPaletteBackgroundColor(paletteBackgroundColor().light(160));
	}
	
	l_text->setPaletteForegroundColor( margin_left->paletteForegroundColor().dark(350) ); // margin_left przykadowo - moe by kady inny poza text
	
	kdebugf2();
}

void OSDWidget::leave(void)
{
	kdebugf();
	
	if( m_translucency )
	{
		margin_left->setPaletteBackgroundPixmap(background_l);
		l_pixmap->setPaletteBackgroundPixmap(background_p);
		margin_center->setPaletteBackgroundPixmap(background_c);
		l_text->setPaletteBackgroundPixmap(background_t);
		margin_right->setPaletteBackgroundPixmap(background_r);
	}
	else
	{
		margin_left->setPaletteBackgroundColor(paletteBackgroundColor());
		l_pixmap->setPaletteBackgroundColor(paletteBackgroundColor());
		margin_center->setPaletteBackgroundColor(paletteBackgroundColor());
		l_text->setPaletteBackgroundColor(paletteBackgroundColor());
		margin_right->setPaletteBackgroundColor(paletteBackgroundColor());
	}
	
	l_text->setPaletteForegroundColor( margin_left->paletteForegroundColor() ); // margin_left przykadowo - moe by kady inny poza text
	
	kdebugf2();
}

void OSDWidget::unsetColors(void)
{
	kdebugf();
	
	const QColorGroup c = QApplication::palette().active();
	
	setPaletteBackgroundColor( c.highlight() );
	setPaletteForegroundColor( c.highlightedText() );
	
	kdebugf2();
}

void OSDWidget::timeout(bool chat)
{
	hide();
	emit timeout(m_id, chat);
}

bool OSDWidget::eventFilter(QObject *obj, QEvent *event)
{
	kdebugf();
// 0 - nothing
// 1 - open chat
// 2 - delete hint
// 3 - delete all hints
	
	switch (event->type())
	{
		case QEvent::Enter:
			enter();
			return true;
			break;
		case QEvent::Leave:
			leave();
			return true;
			break;

		case QEvent::MouseButtonRelease:
		{
			switch ((dynamic_cast<QMouseEvent*>(event))->button())		// co to za tasiemiec?
			{
				case Qt::MidButton:
					switch(config_file.readNumEntry("osdhints", "MiddleButton"))
					{
						case 1:
							if (!m_uin.isEmpty())
							{
								chat_manager->openPendingMsgs(m_uin);
								timeout(true);
							}
							break;
						case 2:
							timeout(false);
							break;
						case 3:
							emit deleteAll();
							break;
						default:
							break;
					}
					break;
				
				
				case Qt::RightButton:
					switch(config_file.readNumEntry("osdhints", "RightButton"))
					{
						case 1:
							if (!m_uin.isEmpty())
							{
								chat_manager->openPendingMsgs(m_uin);
								timeout(true);
							}
							break;
						case 2:
							timeout(false);
							break;
						case 3:
							emit deleteAll();
							break;
						default:
							break;
					}
					break;
				
//				case Qt::LeftButton:
				default:
					switch(config_file.readNumEntry("osdhints", "LeftButton"))
					{
						case 1:
							if (!m_uin.isEmpty())
							{
								chat_manager->openPendingMsgs(m_uin);
								timeout(true);
							}
							break;
						case 2:
// 							if(config_file.readBoolEntry("osdhints", "DeletePendingMsg", false))
// 								chat_manager->deletePendingMsgs(m_uin);
							timeout(false);
							break;
						case 3:
							emit deleteAll();
							break;
						default:
							break;
					}
					break;
			}
		}
		
		default:
			return false;
			break;
	}
	
	kdebugf2();
}

//////  OSDPreviewWidget below /////////////////////

#include <qcursor.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include "misc.h"

OSDPreviewWidget::OSDPreviewWidget( QWidget *parent )
        : OSDWidget( parent, "kadu_osd_notify_preview" )
        , m_dragging( false )
{
	kdebugf();
	
	m_text = tr( "OSD Preview - drag to reposition<BR><small>Translucency may not work fine now.</small>" );
	m_duration = 0;
	
	kdebugf2();
}

bool OSDPreviewWidget::eventFilter(QObject *obj, QEvent *event)
{
	kdebugf();

	switch (event->type())
	{
		case QEvent::Enter:
			enter();
			return true;
			break;
		case QEvent::Leave:
			leave();
			return true;
			break;
		
		case QEvent::MouseMove:
		{
			if( m_dragging && this == mouseGrabber() )
			{
				const QRect screen = QApplication::desktop()->screenGeometry( 0 );
			
				QPoint destination = (dynamic_cast<QMouseEvent*>(event))->globalPos() - m_dragOffset - screen.topLeft();
		// czy nie wyszed za ekran
				int maxX = screen.width() - width();
				if( destination.x() < 0 ) destination.rx() = 0;
				if( destination.x() > maxX ) destination.rx() = maxX;
				int maxY = screen.height() - height();
				if( destination.y() < 0 ) destination.ry() = 0;
				if( destination.y() > maxY ) destination.ry() = maxY;
		
		
				destination += screen.topLeft();
		
				move( destination );
			}
			return true;
			break;
		}
		
		case QEvent::MouseButtonRelease:
		{
			if( m_dragging )
			{
				m_dragging = false;
				releaseMouse();
			
				// compute current Position && offset
				QDesktopWidget *desktop = QApplication::desktop();
				int currentScreen = desktop->screenNumber( pos() );
			
				if( currentScreen != -1 )
				{
				// set new data
					
					switch ((int)ConfigDialog::getVButtonGroup("osdhints", "For corner:")->selectedId())
					{
						case 1:
							m_x      = QWidget::x();
							m_y      = QWidget::y() + m_height;
							break;
						case 2:
							m_x      = QWidget::x() + m_width;
							m_y      = QWidget::y();
							break;
						case 3:
							m_x      = QWidget::x() + m_width;
							m_y      = QWidget::y() + m_height;
							break;
						default:
							m_x      = QWidget::x();
							m_y      = QWidget::y();
					}
		
					emit positionChanged(m_x, m_y);
				}
			}
			return true;
			break;
		}

		case QEvent::MouseButtonPress:
		{
			m_dragOffset = (dynamic_cast<QMouseEvent*>(event))->pos();
			
			if( (dynamic_cast<QMouseEvent*>(event))->button() == LeftButton && !m_dragging )
			{
				grabMouse( QCursor::sizeAllCursor );
				m_dragging = true;
			}
			return true;
			break;
		}
		
		default:
			return false;
			break;
	}
	
	kdebugf2();
}

void OSDPreviewWidget::doUpdate()
{
	kdebugf();
	
	hide();
	
	setFont(ConfigDialog::getSelectFont("osdhints", "Font in OSD")->font());
	m_translucency = ConfigDialog::getCheckBox("osdhints", "Make the background translucent")->isChecked();
	m_translucency_level = ConfigDialog::getSpinBox("osdhints", "Translucency level")->value() * 0.01;	// * 0.01, bo w spinie s wartoci w %
	m_x = ConfigDialog::getSpinBox("osdhints", "X:")->value();
	m_y = ConfigDialog::getSpinBox("osdhints", "Y:")->value();
	
	if (ConfigDialog::getCheckBox("osdhints", "Use default colors")->isChecked())
		unsetColors();

	else
	{
		setPaletteBackgroundColor(ConfigDialog::getColorButton("osdhints", "Background color:")->color());
		setPaletteForegroundColor(ConfigDialog::getColorButton("osdhints", "Text color:")->color());
	}
	
	m_pixmap = icons_manager.loadIcon(ConfigDialog::getLineEdit("osdhints", "Path to icon:")->text());
	
	show();
	QWidget::show();

	kdebugf2();
}

QImage OSDWidget::fade(QImage img, const float val, const QColor &color)
/**

Code copied from kpixmapeffect.cpp (GPL)

**/

{
	if (img.width() == 0 || img.height() == 0)
		return img;
	
	// We don't handle bitmaps
	if (img.depth() == 1)
		return img;
	
	unsigned char tbl[256];
	for (int i=0; i<256; i++)
		tbl[i] = (int) (val * i + 0.5);
	
	int red = color.red();
	int green = color.green();
	int blue = color.blue();
	
	QRgb col;
	int r, g, b, cr, cg, cb;
	
	if (img.depth() <= 8)
	{
		// pseudo color
		for (int i=0; i<img.numColors(); i++)
		{
			col = img.color(i);
			cr = qRed(col); cg = qGreen(col); cb = qBlue(col);
			if (cr > red)
				r = cr - tbl[cr - red];
			else
				r = cr + tbl[red - cr];
			if (cg > green)
				g = cg - tbl[cg - green];
			else
				g = cg + tbl[green - cg];
			if (cb > blue)
				b = cb - tbl[cb - blue];
			else
				b = cb + tbl[blue - cb];
			img.setColor(i, qRgba(r, g, b, qAlpha(col)));
		}
	} else 
	{
	// truecolor
		for (int y=0; y<img.height(); y++)
		{
			QRgb *data = (QRgb *) img.scanLine(y);
			for (int x=0; x<img.width(); x++)
			{
				col = *data;
				cr = qRed(col); cg = qGreen(col); cb = qBlue(col);
				if (cr > red)
					r = cr - tbl[cr - red];
				else
					r = cr + tbl[red - cr];
				if (cg > green)
					g = cg - tbl[cg - green];
				else
					g = cg + tbl[green - cg];
				if (cb > blue)
					b = cb - tbl[cb - blue];
				else
					b = cb + tbl[blue - cb];
				*data++ = qRgba(r, g, b, qAlpha(col));
			}
		}
	}
	
	return img;
}

