#include <qpixmap.h>
#include <qimage.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qapplication.h>

#include "misc.h" //UinsList

#include <qhbox.h> //baseclass

extern QColor def_text_color;
extern QColor def_bg_color;

class OSDWidget : public QHBox
{
	Q_OBJECT
	
	public:
		OSDWidget(QWidget *parent, const char *name = "kadu_osd_notify");

		unsigned int	m_width;
		unsigned int	m_height;
		int		m_x;
		int		m_y;
		UinsList 	m_uin;
		
	public slots:
		void show();
		
		void setTextColor( const QColor &color ) { setPaletteForegroundColor( color ); }
		void setBackgroundColor(const QColor &color ) { setPaletteBackgroundColor( color ); }
		void setDuration( int ms ) { m_duration = ms; }
		void setPixmap( const QPixmap &pixmap ) { m_pixmap = pixmap; }
		void setText( const QString &text ) { m_text = text; }
		void setTranslucency( bool b ) { m_translucency = b; }
		void setTranslucencyLevel( double v ) { m_translucency_level = v; }
		void setX( int x ) { m_x = x; }
		void setY( int y ) { m_y = y; }
		void setUin( UinsList uin ) { m_uin = uin; }
		void setID( unsigned int id ) { m_id = id; }
	
	protected:
		bool eventFilter(QObject *obj, QEvent *event);

	private:
		void render( const uint marginMetric, const QRect &rect );
		QRect determineMetrics(unsigned int);
		QImage fade(QImage, const float, const QColor &);
		void unsetColors();
		void enter();	//rozjania dymek (to)
		void leave();	//ustawia domylny kolor ta
		
		QWidget		*margin_left;
		QLabel		*l_pixmap;
		QWidget		*margin_center;
		QLabel		*l_text;
		QWidget		*margin_right;
		
		QPixmap		m_pixmap;
		QString		m_text;
		
		int		m_duration;
		QTimer		*m_timer;
		bool		m_translucency;
		double		m_translucency_level;
		unsigned int 	m_id;
		
		// ta
		QPixmap background_l;
		QPixmap background_p;
		QPixmap background_c;
		QPixmap background_t;
		QPixmap background_r;
		
		// ta rozjanione (light) - pojawiaj si po najechaniu kursorem
		QPixmap background_l_l;
		QPixmap background_p_l;
		QPixmap background_c_l;
		QPixmap background_t_l;
		QPixmap background_r_l;
		
	private slots:
		void timeout(bool chat = false);
		
	signals:
		void timeout(unsigned int, bool);
		void deleteAll(void);
		
	friend class OSDPreviewWidget;
};

class OSDPreviewWidget : public OSDWidget
{
	Q_OBJECT
	
	public:
		OSDPreviewWidget( QWidget *parent );
	
	public slots:
		// if, eby nie wywoywa si przy przesuniciu myszk
		void setX( int NewX ) {if (NewX != m_x) {m_x = NewX; QWidget::hide(); show(); QWidget::show(); }}
		void setY( int NewY ) {if (NewY != m_y) {m_y = NewY; QWidget::hide(); show(); QWidget::show(); }}
		void doUpdate();
		
	protected:
		bool eventFilter(QObject *obj, QEvent *event);

	private:
		bool   m_dragging;
		QPoint m_dragOffset;

	signals:
		void positionChanged(unsigned int, unsigned int);
};
