/***************************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

***************************************************************************
Copyright (C) 2005 by 
Pan Wojtas (Wojtek Sulewski)
wojteksulewski <at> op.pl
gg: 2087202

***************************************************************************/

#include <qspinbox.h>
#include <qcheckbox.h>
#include <qvgroupbox.h>
#include <qvbuttongroup.h>
#include <qcombobox.h>

#include "../notify/notify.h"
#include "config_dialog.h"
#include "misc.h"
#include "debug.h"

#include "osdhints_notify.h"


OSD_Notify *osdhints_notify_module;

extern "C" int osdhints_notify_init()
{
	kdebugf();
	
	osdhints_notify_module = new OSD_Notify();
	
	kdebugf2();
	return 0;
}

extern "C" void osdhints_notify_close()
{
	kdebugf();
	
	delete osdhints_notify_module;

	kdebugf2();
}

OSD_Notify::OSD_Notify()
{
	kdebugf();

	createGUI();

// dodaje do notify
	QMap<QString, QString> s;
	s["NewChat"] = SLOT(newChat(const UinsList &, const QString &, time_t));
	s["NewMessage"] = SLOT(newMessage(const UinsList &, const QString &, time_t, bool &));
	s["ConnError"] = SLOT(connectionError(const QString &));
	s["toAvailable"] = SLOT(userChangedStatusToAvailable(const UserListElement &));
	s["toBusy"] = SLOT(userChangedStatusToBusy(const UserListElement &));
	s["toInvisible"] = SLOT(userChangedStatusToInvisible(const UserListElement &));
	s["toNotAvailable"] = SLOT(userChangedStatusToNotAvailable(const UserListElement &));
	s["Message"] = SLOT(message(const QString &, const QString &, const QMap<QString, QVariant> *, const UserListElement *));

	osd_manager = new OSDManager();
	notify->registerNotifier(QT_TRANSLATE_NOOP("@default", "osdhints"), osd_manager, s);
	
	
	kdebugf2();
}

OSD_Notify::~OSD_Notify()
{
	kdebugf();

	notify->unregisterNotifier("osdhints");

	destroyGUI();
	
	delete osd_manager;
	
	kdebugf2();
}

void OSD_Notify::createGUI(void)
{
	kdebugf();
// podgld
	preview = new OSDPreviewWidget(0);
	connect(preview, SIGNAL(positionChanged(unsigned int, unsigned int)), this, SLOT(preview_positionChanged(unsigned int, unsigned int)));

// nowy tab w konfiguracji

ConfigDialog::addTab(QT_TRANSLATE_NOOP("@default", "osdhints"), dataPath("kadu/modules/data/osdhints_notify/osd_icon_alt.png"));

	ConfigDialog::addVGroupBox("osdhints", "osdhints", QT_TRANSLATE_NOOP("@default", "New chat / new message"));
		ConfigDialog::addCheckBox("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Show message content in hint"), "ShowContentMessage", true);
//	ConfigDialog::addCheckBox("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Delete pending message when hint deleted"), "DeletePendingMsg", false);
// 		ConfigDialog::addCheckBox("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "pipiq"), "pipiw", false, QString(""), "pipip");
		ConfigDialog::addSpinBox("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Number of quoted characters"), "CiteSign", 10, 1000, 1, 50);
		ConfigDialog::addCheckBox("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Use custom syntax"), "UseSyntax_chat", false, QString(""), "syntax_1");
		ConfigDialog::addLineEdit("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Hint syntax"), "Syntax_chat", "", Kadu::SyntaxText, "syntax_line_1");
		ConfigDialog::addCheckBox("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Show photo if exists (instead of an icon)"), "UsePhoto_chat", false, QString(""), "photo_1");
		ConfigDialog::addLineEdit("osdhints", "New chat / new message", QT_TRANSLATE_NOOP("@default", "Photo path syntax"), "PhotoSyntax_chat", "photos/%a.jpg", Kadu::SyntaxText, "photo_syntax_1");
		
	ConfigDialog::addVGroupBox("osdhints", "osdhints", QT_TRANSLATE_NOOP("@default", "Status change"));
		ConfigDialog::addCheckBox("osdhints", "Status change", QT_TRANSLATE_NOOP("@default", "Open chat on click"), "OpenChatOnClick", false);
		ConfigDialog::addCheckBox("osdhints", "Status change", QT_TRANSLATE_NOOP("@default", "Use custom syntax"), "UseSyntax_status", false, QString(""), "syntax_2");
		ConfigDialog::addLineEdit("osdhints", "Status change", QT_TRANSLATE_NOOP("@default", "Hint syntax"), "Syntax_status", "", Kadu::SyntaxText, "syntax_line_2");
		ConfigDialog::addCheckBox("osdhints", "Status change", QT_TRANSLATE_NOOP("@default", "Show photo if exists (instead of an icon)"), "UsePhoto_status", false, QString(""), "photo_2");
		ConfigDialog::addLineEdit("osdhints", "Status change", QT_TRANSLATE_NOOP("@default", "Photo path syntax"), "PhotoSyntax_status", "photos/%a.jpg", Kadu::SyntaxText, "photo_syntax_2");

	ConfigDialog::addVGroupBox("osdhints", "osdhints", QT_TRANSLATE_NOOP("@default", "Mouse buttons"));
		QStringList options=toStringList(tr("Nothing"),tr("Open chat"),tr("Delete hint"),tr("Delete all hints"));
		QStringList values=toStringList("0","1","2","3");
		ConfigDialog::addComboBox("osdhints", "Mouse buttons", QT_TRANSLATE_NOOP("@default", "Left button"), "LeftButton", options, values, "2");
		ConfigDialog::addComboBox("osdhints", "Mouse buttons", QT_TRANSLATE_NOOP("@default", "Middle button"), "MiddleButton", options, values, "3");
		ConfigDialog::addComboBox("osdhints", "Mouse buttons", QT_TRANSLATE_NOOP("@default", "Right button"), "RightButton", options, values, "1");

	ConfigDialog::addHGroupBox("osdhints", "osdhints", QT_TRANSLATE_NOOP("@default", "Translucency"));
		ConfigDialog::addCheckBox("osdhints", "Translucency", QT_TRANSLATE_NOOP("@default", "Make the background translucent"), "Use translucency", false);
		ConfigDialog::addSpinBox("osdhints", "Translucency", QT_TRANSLATE_NOOP("@default", "Translucency level"), "Translucency level", 0, 100, 1, 80);

	ConfigDialog::addHBox("osdhints", "osdhints", "hints-hbox");
		ConfigDialog::addVRadioGroup("osdhints", "hints-hbox", QT_TRANSLATE_NOOP("@default", "New OSD goes"), "NewOSD",
			     toStringList(tr("On top"),tr("Above"),tr("On bottom"),tr("Below"),tr("On the same place")),
			     toStringList("0","1","2","3","4"),"0");
			ConfigDialog::addSpinBox("osdhints", "New OSD goes", "Separator", "OSD_separator", 0, 50, 1, 1);
		ConfigDialog::addVGroupBox("osdhints", "hints-hbox", QT_TRANSLATE_NOOP("@default", "Position"));
			ConfigDialog::addVRadioGroup("osdhints", "Position", QT_TRANSLATE_NOOP("@default", "For corner:"), "Corner",
			toStringList(tr("Left top"), tr("Left bottom"), tr("Right top"), tr("Right bottom")),
			toStringList("0", "1", "2", "3"),"3");
			ConfigDialog::addSpinBox("osdhints", "Position", QT_TRANSLATE_NOOP("@default", "X:"), "OSD_x", 0, 3000, 1, 600);
			ConfigDialog::addSpinBox("osdhints", "Position", QT_TRANSLATE_NOOP("@default", "Y:"), "OSD_y", 0, 3000, 1, 400);
	
	
	
	/* poszczeglne typy */


	ConfigDialog::addVGroupBox("osdhints", "osdhints", QT_TRANSLATE_NOOP("@default", "Parameters"));
	ConfigDialog::addCheckBox("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "Set for all"), "SetAll", true);

		QStringList options2;
		QStringList values2;
		options2<<tr("Online")<<tr("Online (d.)")<<tr("Busy")<<tr("Busy (d.)")<<
				tr("Invisible")<<tr("Invisible (d.)")<<tr("Offline")<<tr("Offline (d.)")<<
				tr("New chat")<<tr("New message in chat")<<tr("Error")<<tr("Other message");
		values2<<"0"<<"1"<<"2"<<"3"<<"4"<<"5"<<"6"<<"7"<<"8"<<"9"<<"10"<<"11";
		ConfigDialog::addComboBox("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "OSD type"), "LastSelected", options2, values2, "0");

		ConfigDialog::addVGroupBox("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "Fonts"));
			ConfigDialog::addSelectFont("osdhints", "Fonts", QT_TRANSLATE_NOOP("@default", "Font in OSD"), "a");

		ConfigDialog::addCheckBox("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "Use default colors"), "aa", true);
		ConfigDialog::addVGroupBox("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "Colors"));
			ConfigDialog::addColorButton("osdhints", "Colors", QT_TRANSLATE_NOOP("@default", "Background color:"), "aaa", QColor("#000000"));
			ConfigDialog::addColorButton("osdhints", "Colors", QT_TRANSLATE_NOOP("@default", "Text color:"), "aaaa", QColor("#000000"));

		ConfigDialog::addSpinBox("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "Duration (0 - stay forever)"), "LastTimeout", 0, 100000, 100, 3000);
		ConfigDialog::addLineEdit("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default", "Path to icon:"), "aaaaa", "");
		ConfigDialog::addPushButton("osdhints", "Parameters", QT_TRANSLATE_NOOP("@default","Choose path"));


	config_opts_prefixes<<"OSDOnline"<<"OSDOnlineD"<<"OSDBusy"<<"OSDBusyD"<<
					"OSDInvisible"<<"OSDInvisibleD"<<"OSDOffline"<<"OSDOfflineD"<<
					"OSDNewChat"<<"OSDNewMessage"<<"OSDError"<<"OSDMessage";
					
		
	config_hint_properties.clear();

	CONST_FOREACH(prefix, config_opts_prefixes)
	{
		OSDProperties prop;
		QFont font(defaultFont->family(), 20);		// def. font
		prop.font=config_file.readFontEntry("osdhints", (*prefix)+"_font", &font);
		prop.fgcolor=config_file.readColorEntry("osdhints", (*prefix)+"_fgcolor", &def_text_color);
		prop.bgcolor=config_file.readColorEntry("osdhints", (*prefix)+"_bgcolor", &def_bg_color);
		prop.timeout=config_file.readUnsignedNumEntry("osdhints", (*prefix)+"_timeout", 3000);
		prop.def_colors=config_file.readBoolEntry("osdhints", (*prefix)+"_def_colors", true);
		prop.icon=config_file.readEntry("osdhints", (*prefix)+"_icon");
		if (prop.icon == QString::null)
		{
			if ((*prefix) == "OSDOnline")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/online64.png");
			}
			else if ((*prefix) == "OSDOnlineD")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/online_d64.png");
			}
			else if ((*prefix) == "OSDBusy")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/busy64.png");
			}
			else if ((*prefix) == "OSDBusyD")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/busy_d64.png");
			}
			else if ((*prefix) == "OSDInvisible")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/invisible64.png");
			}
			else if ((*prefix) == "OSDInvisibleD")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/invisible_d64.png");
			}
			else if ((*prefix) == "OSDOffline")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/offline64.png");
			}
			else if ((*prefix) == "OSDOfflineD")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/offline_d64.png");
			}
			else if ((*prefix) == "OSDNewChat")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/message64.png");
			}
			else if ((*prefix) == "OSDNewMessage")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/message64.png");
			}
			else if ((*prefix) == "OSDError")
			{
				prop.icon=dataPath("kadu/modules/data/osdhints_notify/connection_err_alt64.png");
			}
/*			else if ((*prefix) == "OSDMessage")
			{
			}
*/			
		}

		config_hint_properties[*prefix]=prop;
	}
	
	ConfigDialog::connectSlot("osdhints", "Show message content in hint", SIGNAL(toggled(bool)), this, SLOT(toggled_ShowContent(bool)));
	ConfigDialog::connectSlot("osdhints", "Use custom syntax", SIGNAL(toggled(bool)), this, SLOT(toggled_CustomSyntax_chat(bool)), "syntax_1");
	ConfigDialog::connectSlot("osdhints", "Use custom syntax", SIGNAL(toggled(bool)), this, SLOT(toggled_CustomSyntax_status(bool)), "syntax_2");
	ConfigDialog::connectSlot("osdhints", "Show photo if exists (instead of an icon)", SIGNAL(toggled(bool)), this, SLOT(toggled_ShowPhoto_chat(bool)), "photo_1");
	ConfigDialog::connectSlot("osdhints", "Show photo if exists (instead of an icon)", SIGNAL(toggled(bool)), this, SLOT(toggled_ShowPhoto_status(bool)), "photo_2");
	ConfigDialog::connectSlot("osdhints", "Make the background translucent", SIGNAL(toggled(bool)), this, SLOT(changeUseTrans(bool)));
	ConfigDialog::connectSlot("osdhints", "Use default colors", SIGNAL(toggled(bool)), this, SLOT(toggled_UseDefColors(bool)));
	ConfigDialog::connectSlot("osdhints", "Set for all", SIGNAL(toggled(bool)), this, SLOT(toggled_SetAll(bool)));
	ConfigDialog::connectSlot("osdhints", "New OSD goes", SIGNAL(clicked(int)), this, SLOT(clicked_NewGoes(int)));
	ConfigDialog::connectSlot("osdhints", "For corner:", SIGNAL(clicked(int)), this, SLOT(clicked_ForCorner(int)));
	ConfigDialog::connectSlot("osdhints", "OSD type", SIGNAL(activated(int)), this, SLOT(clicked_HintType(int)));
	ConfigDialog::connectSlot("osdhints", "Duration (0 - stay forever)", SIGNAL(valueChanged(int)), this, SLOT(changed_Timeout(int)));
	ConfigDialog::connectSlot("osdhints", "Font in OSD", SIGNAL(changed(const char *, const QFont&)), this, SLOT(clicked_ChangeFont(const char *, const QFont&)));
	ConfigDialog::connectSlot("osdhints", "Make the background translucent", SIGNAL(toggled(bool)), preview, SLOT(doUpdate()));
	ConfigDialog::connectSlot("osdhints", "Translucency level", SIGNAL(valueChanged(int)), preview, SLOT(doUpdate()));
	ConfigDialog::connectSlot("osdhints", "X:", SIGNAL(valueChanged(int)), preview, SLOT(setX(int)));
	ConfigDialog::connectSlot("osdhints", "Y:", SIGNAL(valueChanged(int)), preview, SLOT(setY(int)));
	ConfigDialog::connectSlot("osdhints", "Background color:", SIGNAL(changed(const char *, const QColor &)), this, SLOT(clicked_ChangeBgColor(const char *, const QColor &)));
	ConfigDialog::connectSlot("osdhints", "Text color:", SIGNAL(changed(const char *, const QColor &)), this, SLOT(clicked_ChangeFgColor(const char *, const QColor &)));
	ConfigDialog::connectSlot("osdhints", "Choose path", SIGNAL(clicked()), this, SLOT(clicked_ChooseIcon()));
	ConfigDialog::connectSlot("osdhints", "Path to icon:", SIGNAL(textChanged( const QString & )), this, SLOT(changed_PathToIcon( const QString & )));
	
	ConfigDialog::registerSlotsOnTabChange("osdhints", this, SLOT(tabIn()), SLOT(tabOut()));
	ConfigDialog::registerSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::registerSlotOnClose(this, SLOT(tabOut()));
	ConfigDialog::registerSlotOnApply(this, SLOT(onApplyConfigDialog()));
	
	kdebugf2();
}

void OSD_Notify::onCreateConfigDialog(void)
{
	kdebugf();
	bool val = config_file.readBoolEntry("osdhints", "SetAll");
	ConfigDialog::getComboBox("osdhints", "OSD type")->setEnabled(!val);

	currentOptionPrefix.truncate(0);

	currentOptionPrefix=config_opts_prefixes[ConfigDialog::getComboBox("osdhints", "OSD type")->currentItem()];

	ConfigDialog::getSpinBox("osdhints", "Duration (0 - stay forever)")->setSuffix(" ms");
	ConfigDialog::getSpinBox("osdhints", "Translucency level")->setSuffix(" %");
	ConfigDialog::getVGroupBox("osdhints", QT_TRANSLATE_NOOP("@default", "Colors"))->setEnabled(!ConfigDialog::getCheckBox("osdhints", "Use default colors")->isChecked());
	ConfigDialog::getSpinBox("osdhints", "Translucency level")->setEnabled(ConfigDialog::getCheckBox("osdhints", "Make the background translucent")->isChecked());
	ConfigDialog::getSpinBox("osdhints", "Number of quoted characters")->setEnabled(ConfigDialog::getCheckBox("osdhints", "Show message content in hint")->isChecked());
	ConfigDialog::getLineEdit("osdhints", "Hint syntax", "syntax_line_1")->setEnabled(ConfigDialog::getCheckBox("osdhints", "Use custom syntax", "syntax_1")->isChecked());
	ConfigDialog::getLineEdit("osdhints", "Hint syntax", "syntax_line_2")->setEnabled(ConfigDialog::getCheckBox("osdhints", "Use custom syntax", "syntax_2")->isChecked());
	ConfigDialog::getLineEdit("osdhints", "Photo path syntax", "photo_syntax_1")->setEnabled(ConfigDialog::getCheckBox("osdhints", "Show photo if exists (instead of an icon)", "photo_1")->isChecked());
	ConfigDialog::getLineEdit("osdhints", "Photo path syntax", "photo_syntax_2")->setEnabled(ConfigDialog::getCheckBox("osdhints", "Show photo if exists (instead of an icon)", "photo_2")->isChecked());
	ConfigDialog::getSpinBox("osdhints", "Separator")->setSuffix(" px");
	clicked_NewGoes(config_file.readNumEntry("osdhints", "NewOSD"));
	clicked_ForCorner(config_file.readNumEntry("osdhints", "Corner"));
	
	kdebugf2();
}

void OSD_Notify::onApplyConfigDialog(void)
{
	kdebugf();
	CONST_FOREACH(prop, config_hint_properties)
	{
		config_file.writeEntry("osdhints", prop.key()+"_font", prop.data().font);
		config_file.writeEntry("osdhints", prop.key()+"_fgcolor", prop.data().fgcolor);
		config_file.writeEntry("osdhints", prop.key()+"_bgcolor", prop.data().bgcolor);
		config_file.writeEntry("osdhints", prop.key()+"_timeout", (int)prop.data().timeout);
		config_file.writeEntry("osdhints", prop.key()+"_def_colors", (bool)prop.data().def_colors);
		config_file.writeEntry("osdhints", prop.key()+"_icon", prop.data().icon);
	}
	
	// eby nie byo crashu
	osd_manager->deleteAll();
	
	kdebugf2();
}

void OSD_Notify::clicked_HintType(int id)
{
	kdebugf();

	currentOptionPrefix=config_opts_prefixes[id];

	OSDProperties prop=config_hint_properties[currentOptionPrefix];

	ConfigDialog::getSelectFont("osdhints", "Font in OSD")->setFont(prop.font);
	ConfigDialog::getSpinBox("osdhints", "Duration (0 - stay forever)")->setValue(prop.timeout);
	ConfigDialog::getCheckBox("osdhints", "Use default colors")->setChecked(prop.def_colors);
	ConfigDialog::getColorButton("osdhints", "Background color:")->setColor(prop.bgcolor);
	ConfigDialog::getColorButton("osdhints", "Text color:")->setColor(prop.fgcolor);
	ConfigDialog::getLineEdit("osdhints", "Path to icon:")->setText(prop.icon);
	
// nie ma:
	preview->doUpdate();
// bo jest w changed_PathToIcon( const QString &path )
// (bo i tak si zmienia LineEdit)
	
	kdebugf2();
}

void OSD_Notify::toggled_SetAll(bool val)
{
	kdebugf();
	
	if (val)
	{
		CONST_FOREACH(prefix, config_opts_prefixes)
		{
			OSDProperties prop;
			
			prop.font = config_hint_properties[currentOptionPrefix].font;
			prop.fgcolor = config_hint_properties[currentOptionPrefix].fgcolor;
			prop.bgcolor = config_hint_properties[currentOptionPrefix].bgcolor;
			prop.timeout = config_hint_properties[currentOptionPrefix].timeout;
			prop.def_colors = config_hint_properties[currentOptionPrefix].def_colors;
			prop.icon = config_hint_properties[*prefix].icon;		// ikona si nie zmienia!
	
			config_hint_properties[*prefix]=prop;
		}
	}
	
	ConfigDialog::getComboBox("osdhints", "OSD type")->setEnabled(!val);
	
	kdebugf2();
}

void OSD_Notify::clicked_ChangeFont(const char *, const QFont& font)
{
	kdebugf();

	if (ConfigDialog::getCheckBox("osdhints", "Set for all")->isChecked())
		FOREACH(prop, config_hint_properties)
			prop.data().font = font;
	else
		config_hint_properties[currentOptionPrefix].font = font;

	preview->doUpdate();
	
	kdebugf2();
}

void OSD_Notify::clicked_ChangeFgColor(const char *, const QColor &value)
{
	kdebugf();

	if (ConfigDialog::getCheckBox("osdhints", "Set for all")->isChecked())
		FOREACH(prop, config_hint_properties)
			prop.data().fgcolor = value;
	else
		config_hint_properties[currentOptionPrefix].fgcolor=value;
	
	preview->doUpdate();

	kdebugf2();
}

void OSD_Notify::clicked_ChangeBgColor(const char *, const QColor &value)
{
	kdebugf();

	if (ConfigDialog::getCheckBox("osdhints", "Set for all")->isChecked())
		FOREACH(prop, config_hint_properties)
			prop.data().bgcolor = value;
	else
		config_hint_properties[currentOptionPrefix].bgcolor=value;

	preview->doUpdate();
	
	kdebugf2();
}

void OSD_Notify::changed_Timeout(int value)
{
	kdebugf();

	if (ConfigDialog::getCheckBox("osdhints", "Set for all")->isChecked())
		FOREACH(prop, config_hint_properties)
			prop.data().timeout = value;
	else
		config_hint_properties[currentOptionPrefix].timeout=value;

	kdebugf2();
}

void OSD_Notify::toggled_ShowContent(bool toggled)
{
	kdebugf();

	ConfigDialog::getSpinBox("osdhints", "Number of quoted characters")->setEnabled(toggled);
	
	kdebugf2();
}

void OSD_Notify::toggled_CustomSyntax_chat(bool toggled)
{
	kdebugf();

	ConfigDialog::getLineEdit("osdhints", "Hint syntax", "syntax_line_1")->setEnabled(toggled);
	
	kdebugf2();
}

void OSD_Notify::toggled_CustomSyntax_status(bool toggled)
{
	kdebugf();

	ConfigDialog::getLineEdit("osdhints", "Hint syntax", "syntax_line_2")->setEnabled(toggled);
	
	kdebugf2();
}

void OSD_Notify::toggled_ShowPhoto_chat(bool toggled)
{
	kdebugf();

	ConfigDialog::getLineEdit("osdhints", "Photo path syntax", "photo_syntax_1")->setEnabled(toggled);
	
	kdebugf2();
}

void OSD_Notify::toggled_ShowPhoto_status(bool toggled)
{
	kdebugf();

	ConfigDialog::getLineEdit("osdhints", "Photo path syntax", "photo_syntax_2")->setEnabled(toggled);
	
	kdebugf2();
}

void OSD_Notify::toggled_UseDefColors(bool value)
{
	kdebugf();
	if (ConfigDialog::getCheckBox("osdhints", "Set for all")->isChecked())
		FOREACH(prop, config_hint_properties)
			prop.data().def_colors = value;
	else
		config_hint_properties[currentOptionPrefix].def_colors=value;
		
	ConfigDialog::getVGroupBox("osdhints", "Colors")->setEnabled(!value);
	preview->doUpdate();
	
	kdebugf2();
}

void OSD_Notify::clicked_ChooseIcon(void)
{
	kdebugf();
	
	QString start=QDir::rootDirPath();
	QString p=ConfigDialog::getLineEdit("osdhints", QT_TRANSLATE_NOOP("@default", "Path to icon:"))->text();

	if (QFile(p).exists())
		start=p;
// TODO: inne formaty?
	QString s(QFileDialog::getOpenFileName( start, "Image Files (*.png)", ConfigDialog::configdialog));
	if (s.length())
		ConfigDialog::getLineEdit("osdhints", QT_TRANSLATE_NOOP("@default", "Path to icon:"))->setText(s);
	
	
// zachowuje ustawienie
	if (ConfigDialog::getCheckBox("osdhints", "Set for all")->isChecked())
		FOREACH(prop, config_hint_properties)
			prop.data().icon = s;
	else
		config_hint_properties[currentOptionPrefix].icon=s;
	
	kdebugf2();
	
}

void OSD_Notify::clicked_NewGoes(int id)
{
	kdebugf();
	
	if(id == 4)
		ConfigDialog::getSpinBox("osdhints", "Separator")->setEnabled(false);
	else
		ConfigDialog::getSpinBox("osdhints", "Separator")->setEnabled(true);
	
	kdebugf2();
	
}

void OSD_Notify::clicked_ForCorner(int id)
{
	kdebugf();
	
	const QRect screen = QApplication::desktop()->screenGeometry( 0 );
	const unsigned int width = preview->m_width;
	const unsigned int height = preview->m_height;

	switch(id)
	{
		case 0:
			ConfigDialog::getSpinBox("osdhints", "X:")->setMaxValue(screen.width() - width);
			ConfigDialog::getSpinBox("osdhints", "Y:")->setMaxValue(screen.height() - height);
			break;
		case 1:
			ConfigDialog::getSpinBox("osdhints", "X:")->setMaxValue(screen.width() - width);
			ConfigDialog::getSpinBox("osdhints", "Y:")->setMaxValue(screen.height());
			break;
		case 2:
			ConfigDialog::getSpinBox("osdhints", "X:")->setMaxValue(screen.width());
			ConfigDialog::getSpinBox("osdhints", "Y:")->setMaxValue(screen.height() - height);
			break;
		default:
			ConfigDialog::getSpinBox("osdhints", "X:")->setMaxValue(screen.width());
			ConfigDialog::getSpinBox("osdhints", "Y:")->setMaxValue(screen.height());
			break;
	}

	kdebugf2();
	
}

void OSD_Notify::changed_PathToIcon( const QString &path )
{
	kdebugf();
// ikona zmienia si tylko dla aktualnego!
		config_hint_properties[currentOptionPrefix].icon=path;
	
	preview->doUpdate();
	
	kdebugf2();
}

void OSD_Notify::changeUseTrans(bool toggled)
{
	kdebugf();
	ConfigDialog::getSpinBox("osdhints", "Translucency level")->setEnabled(toggled);
	
	kdebugf2();
}

void OSD_Notify::preview_positionChanged(unsigned int x, unsigned int y)
{
	kdebugf();
		
	ConfigDialog::getSpinBox("osdhints", "X:")->setValue(x);
	ConfigDialog::getSpinBox("osdhints", "Y:")->setValue(y);
	
	preview->doUpdate();
	
	kdebugf2();
}

void OSD_Notify::tabIn(void)
{
	kdebugf();
	
	clicked_HintType(ConfigDialog::getComboBox("osdhints", "OSD type")->currentItem());
	
	kdebugf2();
}

void OSD_Notify::tabOut(void)
{
	kdebugf();
	
	preview->hide();
	
	kdebugf2();
}

void OSD_Notify::destroyGUI(void)
{
	kdebugf();
	delete preview;
	
	
	ConfigDialog::unregisterSlotsOnTabChange("osdhints", this, SLOT(tabIn()), SLOT(tabOut()));
	ConfigDialog::unregisterSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::unregisterSlotOnClose(this, SLOT(tabOut()));
	ConfigDialog::unregisterSlotOnApply(this, SLOT(onApplyConfigDialog()));

	
	ConfigDialog::disconnectSlot("osdhints", "Show message content in hint", SIGNAL(toggled(bool)), this, SLOT(toggled_ShowContent(bool)));
	ConfigDialog::disconnectSlot("osdhints", "Use custom syntax", SIGNAL(toggled(bool)), this, SLOT(toggled_CustomSyntax_chat(bool)), "syntax_1");
	ConfigDialog::disconnectSlot("osdhints", "Use custom syntax", SIGNAL(toggled(bool)), this, SLOT(toggled_CustomSyntax_status(bool)), "syntax_2");
	ConfigDialog::disconnectSlot("osdhints", "Show photo if exists (instead of an icon)", SIGNAL(toggled(bool)), this, SLOT(toggled_ShowPhoto_chat(bool)), "photo_1");
	ConfigDialog::disconnectSlot("osdhints", "Show photo if exists (instead of an icon)", SIGNAL(toggled(bool)), this, SLOT(toggled_ShowPhoto_status(bool)), "photo_2");
	ConfigDialog::disconnectSlot("osdhints", "Make the background translucent", SIGNAL(toggled(bool)), this, SLOT(changeUseTrans(bool)));
	ConfigDialog::disconnectSlot("osdhints", "Use default colors", SIGNAL(toggled(bool)), this, SLOT(toggled_UseDefColors(bool)));
	ConfigDialog::disconnectSlot("osdhints", "Set for all", SIGNAL(toggled(bool)), this, SLOT(toggled_SetAll(bool)));
	ConfigDialog::disconnectSlot("osdhints", "New OSD goes", SIGNAL(clicked(int)), this, SLOT(clicked_NewGoes(int)));
	ConfigDialog::disconnectSlot("osdhints", "For corner:", SIGNAL(clicked(int)), this, SLOT(clicked_ForCorner(int)));
	ConfigDialog::disconnectSlot("osdhints", "OSD type", SIGNAL(activated(int)), this, SLOT(clicked_HintType(int)));
	ConfigDialog::disconnectSlot("osdhints", "Duration (0 - stay forever)", SIGNAL(valueChanged(int)), this, SLOT(changed_Timeout(int)));
	ConfigDialog::disconnectSlot("osdhints", "Font in OSD", SIGNAL(changed(const char *, const QFont&)), this, SLOT(clicked_ChangeFont(const char *, const QFont&)));
	ConfigDialog::disconnectSlot("osdhints", "Make the background translucent", SIGNAL(toggled(bool)), preview, SLOT(doUpdate()));
	ConfigDialog::disconnectSlot("osdhints", "Translucency level", SIGNAL(valueChanged(int)), preview, SLOT(doUpdate()));
	ConfigDialog::disconnectSlot("osdhints", "X:", SIGNAL(valueChanged(int)), preview, SLOT(setX(int)));
	ConfigDialog::disconnectSlot("osdhints", "Y:", SIGNAL(valueChanged(int)), preview, SLOT(setY(int)));
	ConfigDialog::disconnectSlot("osdhints", "Background color:", SIGNAL(changed(const char *, const QColor &)), this, SLOT(clicked_ChangeBgColor(const char *, const QColor &)));
	ConfigDialog::disconnectSlot("osdhints", "Text color:", SIGNAL(changed(const char *, const QColor &)), this, SLOT(clicked_ChangeFgColor(const char *, const QColor &)));
	ConfigDialog::disconnectSlot("osdhints", "Choose path", SIGNAL(clicked()), this, SLOT(clicked_ChooseIcon()));
	ConfigDialog::disconnectSlot("osdhints", "Path to icon:", SIGNAL(textChanged( const QString & )), this, SLOT(changed_PathToIcon( const QString & )));
	
	
	
	ConfigDialog::removeControl("osdhints", "Choose path");
	ConfigDialog::removeControl("osdhints", "Path to icon:");
	ConfigDialog::removeControl("osdhints", "Duration (0 - stay forever)");
	ConfigDialog::removeControl("osdhints", "Text color:");
	ConfigDialog::removeControl("osdhints", "Background color:");
	ConfigDialog::removeControl("osdhints", "Colors");
	ConfigDialog::removeControl("osdhints", "Use default colors");
	ConfigDialog::removeControl("osdhints", "Font in OSD");
	ConfigDialog::removeControl("osdhints", "Fonts");
	ConfigDialog::removeControl("osdhints", "OSD type");
	ConfigDialog::removeControl("osdhints", "Set for all");
	ConfigDialog::removeControl("osdhints", "Parameters");
	ConfigDialog::removeControl("osdhints", "For corner:");
	ConfigDialog::removeControl("osdhints", "Y:");
	ConfigDialog::removeControl("osdhints", "X:");
	ConfigDialog::removeControl("osdhints", "Separator");
	ConfigDialog::removeControl("osdhints", "Position");
	ConfigDialog::removeControl("osdhints", "New OSD goes");
	ConfigDialog::removeControl("osdhints", "hints-hbox");
	ConfigDialog::removeControl("osdhints", "Translucency level");
	ConfigDialog::removeControl("osdhints", "Make the background translucent");
	ConfigDialog::removeControl("osdhints", "Translucency");
	ConfigDialog::removeControl("osdhints", "Right button");
	ConfigDialog::removeControl("osdhints", "Middle button");
	ConfigDialog::removeControl("osdhints", "Left button");
	ConfigDialog::removeControl("osdhints", "Mouse buttons");
	ConfigDialog::removeControl("osdhints", "Photo path syntax", "photo_syntax_2");
	ConfigDialog::removeControl("osdhints", "Show photo if exists (instead of an icon)", "photo_2");
	ConfigDialog::removeControl("osdhints", "Hint syntax", "syntax_line_2");
	ConfigDialog::removeControl("osdhints", "Use custom syntax", "syntax_2");
	ConfigDialog::removeControl("osdhints", "Open chat on click");
	ConfigDialog::removeControl("osdhints", "Status change");
	ConfigDialog::removeControl("osdhints", "Photo path syntax", "photo_syntax_1");
	ConfigDialog::removeControl("osdhints", "Show photo if exists (instead of an icon)", "photo_1");
	ConfigDialog::removeControl("osdhints", "Use custom syntax", "syntax_1");
	ConfigDialog::removeControl("osdhints", "Hint syntax", "syntax_line_1");
	ConfigDialog::removeControl("osdhints", "Number of quoted characters");
//	ConfigDialog::removeControl("osdhints", "Delete pending message when hint deleted");
	ConfigDialog::removeControl("osdhints", "Show message content in hint");
	ConfigDialog::removeControl("osdhints", "New chat / new message");
	
	ConfigDialog::removeTab("osdhints");
	
	kdebugf2();
}
